/*
loop.c

The main loop. Does: (in this order)

* If needed loads the config files.

* Runs each monitor.

* Gets the output from each monitor.

* Creates the report.

* Sends the report.

* Sleeps and repeats.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

#include "loop.h"
#include "options.h"
#include "monitor.h"
#include "modulefunc.h"
#include "report.h"
#include "sender.h"
#include "configload.h"

int closenow = 0;

int just_dmons = 0;

/* Main loop. */
void mainloop(){
#ifdef HAVE_MALLINFO
  struct mallinfo minfo;
#endif

  RETSIGTYPE signalhandler(int sig);

  /* setup the signal handler */
  signal(SIGINT, signalhandler);
  signal(SIGTERM, signalhandler);
  signal(SIGHUP, signalhandler);
  signal(SIGQUIT, signalhandler);
  signal(SIGPIPE, signalhandler);
  signal(SIGUSR1, signalhandler);

  while (1){

    if (closenow) goto end;

    /* Do we need to load the config files? */
    if (!haveconfig){
      loadconfig(); /* Load config does nothing yet. */
    }

    if (closenow) goto end;
    
    /* Run the monitors. */
    if (!just_dmons)
      runmonitors();

    if (closenow) goto end;

    /* Runs the dmonitors. */
    rundmonitors();

    if (closenow) goto end;

    /* Create the report. */
    createsysreport();

    if (closenow) goto end;

    /* Send the report. */
    sendsysreport();

    if (closenow) goto end;

    if (just_dmons)
      just_dmons = 0;

#ifdef HAVE_MALLINFO
    /* Print memory useage stats. */
    if (!daemon_mode){
      minfo = mallinfo();
      printf("%d bytes arena.\n",minfo.arena);
      printf("%d chunks not in use.\n",minfo.ordblks);
      printf("%d bytes used by free chunks.\n",minfo.fordblks);
      printf("%d bytes allocated by malloc.\n",minfo.uordblks);
      printf("%d bytes used by top-most releaseable chunk.\n",minfo.keepcost);
    }
#endif

    /* In prior versions we slept for updateinterval - monitortimeout - sendertimeout However in this
       version we have wait_module and don't "take the easy way out"
    */
    sleep(updateinterval);

    if (closenow) goto end;

  }

 end:

  /* Cleanly shutdown. */

  /* Kill all dmonitors. */
  killdmonitors();

  /* Delete all temp files. */
  rmalltmpfiles();

  /* Delete the pidfile /var/run/esmd.pid */
  remove("/var/run/esmd.pid");

  exit(EXIT_SUCCESS);

}

/* signal handler */
RETSIGTYPE signalhandler(int sig){
  
  if (!daemon_mode)
    printf("Recived signal %d\n",sig);

  if (sig == SIGPIPE)
    ;
  else if (sig == SIGUSR1)
    just_dmons = 1;
  else
    closenow = 1;
  
  return;

}

static char *id="@(#) $Id: loop.c,v 4.0 1999/10/07 22:56:13 pete Exp $";

/* 
 * $Log: loop.c,v $
 * Revision 4.0  1999/10/07 22:56:13  pete
 * Started work on version 1.0
 *
 * Revision 3.11  1999/09/21 18:21:50  pete
 * Now uses a configure test, HAVE_MALLINFO, to check for the mallinfo
 * function.
 *
 * Revision 3.10  1999/09/21 17:49:22  pete
 * Added memory debugging. Enabled by the DEBUG_MEMORY #define
 *
 * Revision 3.9  1999/09/21 00:55:16  pete
 * Adjusted the sleep code because of the new wait_module function.
 *
 * Revision 3.8  1999/09/20 19:05:42  pete
 * Now calls killdmonitors before shutdown.
 *
 * Revision 3.7  1999/09/20 18:35:19  pete
 * Forgot to only print the "Sleeping..." message while not in daemon mode.
 *
 * Revision 3.6  1999/09/19 21:32:38  pete
 * Fixed a bug with the signalhandler definition. Now compiles with no
 * warnings.
 *
 * Revision 3.5  1999/09/19 18:25:55  pete
 * Added support for automake
 *
 * Revision 3.4  1999/09/19 16:08:16  pete
 * Changed the return type of the signal handler to RETSIGTYPE.
 *
 * Revision 3.3  1999/09/19 14:37:06  pete
 * The SIGUSR1 signal works.
 *
 * Revision 3.2  1999/09/19 14:14:07  pete
 * Added "Sleeping..." message. Starting to add support for SIGUSR1.
 *
 * Revision 3.1  1999/09/19 13:55:50  pete
 * Now will only sleep if updateinterval - monitortimeout - sendertimeout is
 * greater than zero.
 *
 * Revision 3.0  1999/09/19 01:15:23  pete
 * Started work on 0.03
 *
 * Revision 2.1  1999/09/16 23:24:06  pete
 * Now calls rundmonitors
 *
 * Revision 2.0  1999/09/15 16:05:51  pete
 * Started work on version 0.02
 *
 * Revision 1.15  1999/09/12 23:26:29  pete
 * Made mainloop() sleep for updateinterval - monitortimeout - sendertimeout
 * seconds.
 *
 * Revision 1.14  1999/09/12 13:23:26  pete
 * Now ignores SIGPIPEs.
 *
 * Revision 1.13  1999/09/12 12:11:50  pete
 * Now deletes the pid file created in main() - /var/run/esmd.pid
 *
 * Revision 1.12  1999/09/12 01:32:56  pete
 * Signal handling works. No segfaults, just pure clean shutdowns. :)
 *
 * Revision 1.11  1999/09/12 00:44:51  pete
 * Added signal handling. However it's buggy. :( %50 of the time the program
 * segfaults. :(
 *
 * Revision 1.10  1999/09/11 23:09:19  pete
 * Changed close var to closenow
 *
 * Revision 1.9  1999/09/11 22:37:13  pete
 * Added close var.
 *
 * Revision 1.8  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */
