/* The main loop. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#include <sys/time.h>
#include <sys/types.h>
#include <signal.h>
 
#include "main.h"
#include "loop.h"
#include "version.h"
#include "msg.h"

void checkstdin();
void checkfifo(int i);
int forcecheck();

/* The array of fd's for the FIFOs. */
FILE *fifo_fd[ESM_LOG_EMERG];
int fifo_fd2[ESM_LOG_EMERG];

int have_set_alarm = 0;
int forcecheck_num = 0;

void mainloop(){
  RETSIGTYPE signalhandler(int sig);
  struct timeval timeout;
 
  /* The list of files we need to open. */
  char *fifo_name[] = {"emerg","alert","crit","err","warning","notice","info","debug"};

  /* The read array for the select call. */
  fd_set rset;

  int i;
  char *tmpstr = NULL;

  /* Setup the signalhandler to catch SIGALRM */
  signal(SIGALRM, signalhandler);
 
  /* Open the FIFOs we use to communicate with syslogd.
     Note that fopen will block if syslogd isn't running.
     So instead of using fopen we use open with the 
     O_NONBLOCK option. Then we open the stream with fdopen
  */
  for (i = 0; i < ESM_LOG_EMERG; i++){
    tmpstr = strdup(FIFODIR);
    tmpstr = realloc(tmpstr,strlen(tmpstr) + strlen(fifo_name[i]) + 1);
    tmpstr = strcat(tmpstr,fifo_name[i]);

    fifo_fd2[i] = open(tmpstr,O_NONBLOCK | O_RDONLY);

    if (fifo_fd2[i] < 0){
      fifo_fd[i] = NULL;
      free(tmpstr);
      continue;
    }

    fifo_fd[i] = fdopen(fifo_fd2[i],"r");

    free(tmpstr);
  }

  /* Now to the loop. */

  /* The loop is really simple. Since we are a daemon monitor
     we just read from the FIFO's and stdin and do what we need
     to. We use select to block untill a FIFO or stdin needs
     attention.
  */
  while(1){

    /* Setup the select call so we are watching each FIFO and stdin */
    FD_ZERO (&rset);
    FD_SET(fileno(stdin),&rset);
    for (i = 0; i < ESM_LOG_EMERG; i++){
      if (fifo_fd2[i] >= 0)
	FD_SET (fifo_fd2[i], &rset);
    }

    timeout.tv_sec = forcecheck_interval;
    timeout.tv_usec = 0;
    select (FD_SETSIZE,&rset, NULL, NULL,&timeout);

    if (forcecheck_num > 0)
      forcecheck_num--;

    /* Check stdin for input. */
    if (FD_ISSET (fileno(stdin), &rset))
      checkstdin();

    /* Check the FIFOs for input. */
    for (i = 0; i < ESM_LOG_EMERG; i++){
      if (fifo_fd2[i] < 0)
	continue;

      if (FD_ISSET (fifo_fd2[i], &rset)){
	checkfifo(i);
      }
    }
    fflush(NULL);
  }
}

/* Checks stdin for input and does whatever it needs to. */
void checkstdin(){
  char tmpline[6];
  char *tmpline2 = NULL;
  struct syslogmsg *curmsg = NULL,*prevmsg = NULL;

  tmpline2 = fgets(tmpline,5,stdin);

  if (!tmpline2)
    return;

  if (!strncmp(tmpline2,"STAT",4)){
    printf("BEGIN\n");
    printf("URGENCY %d\n",maxurgency);
    printf("REPORT\n");

    /* Are there any msg in the msg list? */
    if (syslogmsglist){

      /* Print all of the msgs in the msg list. */
      curmsg = syslogmsglist;
      while (1){
	printf("%d: %s",curmsg->urgency,curmsg->msg);
	prevmsg = curmsg;
	if (curmsg->next)
	  curmsg = curmsg->next;
	else{
	  free(prevmsg->msg);
	  free(prevmsg);
	  break;
	}
	free(prevmsg->msg);
	free(prevmsg);
      }

      maxurgency = 0;
      nummsgs = 0;
      syslogmsglist = NULL;
    }
    else
      printf("No messages.\n");

    printf("DONE\n");

  }
}

/* Checks FIFO i */
void checkfifo(int i){
  char tmpline[1024];
  char *tmpline2 = NULL;
  struct syslogmsg *newmsg;

  tmpline2 = fgets(tmpline,1024,fifo_fd[i]);

  if (!tmpline2)
    return;

  /* Set maxurgency. */
  if (maxurgency < ESM_LOG_EMERG - i)
    maxurgency = ESM_LOG_EMERG - i;

  /* If we should notify esmd do so in 1 minute. Since we may end up
   never running forcecheck if we get repeated notifies we only set
   the alarm if we havn't already done so. */
  if ((ESM_LOG_EMERG - i) >= notifylevel){
    if (!have_set_alarm){
      have_set_alarm = 1;
      alarm(60);
    }
  }

  /* Add the message to the message list. */
  newmsg = malloc(sizeof(struct syslogmsg));
  newmsg->msg = strdup(tmpline2);
  newmsg->urgency = ESM_LOG_EMERG - i; /* The urgency is reversed. Have to fix that. */

  addmsg(newmsg);
}

/* Sends SIGUSR1 to esmd. Returns 0 on success, non-zero on fail. */
int forcecheck(){
  FILE *pidfile = NULL;
  int pid,i;

  /* Have we already been run? */
  if (forcecheck_num > 0)
    return 0;

  pidfile = fopen("/var/run/esmd.pid","r");
  
  if (!pidfile)
    return 1;
  
  i = fscanf(pidfile,"%d",&pid);

  fclose(pidfile);

  if (i != 1)
    return 1;

  if (pid <= 0)
    return 1;

  return kill(pid,SIGUSR1);
}

/* signal handler. Only used by SIGALRM */
RETSIGTYPE signalhandler(int sig){

  forcecheck();

  have_set_alarm = 0;

  return;

}

static char *id="@(#) $Id: loop.c,v 2.1 1999/10/07 23:39:04 pete Exp $";

/*
 * $Log: loop.c,v $
 * Revision 2.1  1999/10/07 23:39:04  pete
 * Now prints the urgency of the message before the message.
 *
 * Revision 2.0  1999/10/07 23:26:50  pete
 * Started work on version 1.1
 *
 * Revision 1.18  1999/09/21 21:34:29  pete
 * Changed the forcecheck interval code.
 *
 * Revision 1.17  1999/09/21 21:01:54  pete
 * Changed the forcecheck wait to 5 minutes.
 *
 * Revision 1.16  1999/09/21 20:56:30  pete
 * Now waits 60 seconds before calling forcecheck so multiple messages won't
 * trigger huge numbers of system reports.
 *
 * Revision 1.15  1999/09/21 20:35:40  pete
 * Added support for emergency notification, notifylevel.
 *
 * Revision 1.14  1999/09/21 01:44:01  pete
 * Removed newmsg->next = NULL line. The idea is that addmsg should do all
 * of that for us.
 *
 * Revision 1.13  1999/09/21 01:42:01  pete
 * Moved msglist stuff to msg.c and msg.h
 *
 * Revision 1.12  1999/09/21 01:23:49  pete
 * Now uses addmsg to add msgs.
 *
 * Revision 1.11  1999/09/20 18:42:48  pete
 * Ooops! Wrote HAVE_FCNTL instead of HAVE_FCNTL_H
 *
 * Revision 1.10  1999/09/20 18:25:53  pete
 * Added some config.h stuff for unistd and fcntl
 *
 * Revision 1.9  1999/09/20 18:16:44  pete
 * Now works perfectly when stdin and stdout are FIFO's.
 *
 * Revision 1.8  1999/09/20 17:41:36  pete
 * Fixed a few bugs.
 *
 * Revision 1.7  1999/09/20 17:22:09  pete
 * Works perfectly.
 *
 * Revision 1.6  1999/09/20 16:58:54  pete
 * Added code to reset maxurgency and nummsgs to checkstdin
 *
 * Revision 1.5  1999/09/20 16:56:30  pete
 * The checkstdin function works.
 *
 * Revision 1.4  1999/09/20 15:26:46  pete
 * Select works. Both stdin and the FIFOs are correctly checked for input.
 *
 * Revision 1.3  1999/09/20 15:12:50  pete
 * And now things compile. :)
 *
 * Revision 1.2  1999/09/20 15:10:35  pete
 * Got the opening of the files done. The select setup is done. Now to make
 * the loop.
 *
 * Revision 1.1  1999/09/20 14:23:46  pete
 * First version.
 *
 */
